"""
Money unittests as mixins for Money and subclasses
"""
import abc
from decimal import Decimal
import collections
import unittest

from money import Money, XMoney


class ClassMixin(object):
    def test_new_instance_int_amount(self):
        self.assertIsInstance(self.MoneyClass(0, 'XXX'), self.MoneyClass)
        self.assertIsInstance(self.MoneyClass(12345, 'XXX'), self.MoneyClass)
    
    def test_new_instance_decimal_amount(self):
        self.assertIsInstance(self.MoneyClass(Decimal(12345), 'XXX'), self.MoneyClass)
    
    def test_new_instance_float_amount(self):
        self.assertIsInstance(self.MoneyClass(12345.12345, 'XXX'), self.MoneyClass)
    
    def test_new_instance_str_amount(self):
        self.assertIsInstance(self.MoneyClass('0', 'XXX'), self.MoneyClass)
        self.assertIsInstance(self.MoneyClass('12345.12345', 'XXX'), self.MoneyClass)
    
    def test_invalid_currency_none(self):
        with self.assertRaises(ValueError):
            money = self.MoneyClass('2.22', None)
    
    def test_invalid_currency_false(self):
        with self.assertRaises(ValueError):
            money = self.MoneyClass('2.22', False)
    
    def test_invalid_currency_empty(self):
        with self.assertRaises(ValueError):
            money = self.MoneyClass('2.22', '')
    
    def test_invalid_currency_code(self):
        with self.assertRaises(ValueError):
            money = self.MoneyClass('2.22', 'XX')
        with self.assertRaises(ValueError):
            money = self.MoneyClass('2.22', '123')
        with self.assertRaises(ValueError):
            money = self.MoneyClass('2.22', 'xxx')
        with self.assertRaises(ValueError):
            money = self.MoneyClass('2.22', '$')
        with self.assertRaises(ValueError):
            money = self.MoneyClass('2.22', 'US$')
    
    def test_invalid_amount(self):
        with self.assertRaises(ValueError):
            money = self.MoneyClass('twenty', 'XXX')
    
    def test_not_hashable(self):
        money = self.MoneyClass('2.22', 'XXX')
        self.assertFalse(isinstance(money, collections.Hashable))


class RepresentationsMixin(object):
    def test_repr(self):
        self.assertEqual(repr(self.MoneyClass('1234.567', 'XXX')), 'XXX 1234.567')
    
    def test_str(self):
        self.assertEqual(str(self.MoneyClass('1234.567', 'XXX')), 'XXX 1,234.57')


class FormattingMixin(object):
    def setUp(self):
        self.money = self.MoneyClass('-1234.567', 'USD')
    
    def test_custom_format_padding(self):
        self.assertEqual(self.money.format('en_US', '¤000000.00'), '-$001234.57')
    
    def test_custom_format_custom_negative(self):
        self.assertEqual(self.money.format('en_US', '¤#,##0.00;<¤#,##0.00>'), '<$1,234.57>')
    
    def test_custom_format_grouping(self):
        self.assertEqual(self.money.format('en_US', '¤#,##0.00'), '-$1,234.57')
        self.assertEqual(self.money.format('de_DE', '#,##0.00 ¤'), '-1.234,57 $')
        self.assertEqual(self.money.format('en_US', '¤0.00'), '-$1234.57')
        self.assertEqual(self.money.format('de_DE', '0.00 ¤'), '-1234,57 $')
    
    def test_custom_format_decimals(self):
        self.assertEqual(self.money.format('en_US', '¤0.000'), '-$1234.567')
        self.assertEqual(self.money.format('en_US', '¤0'), '-$1235')
    
    def test_auto_format_locales(self):
        self.assertEqual(self.money.format('en_US'), '($1,234.57)')
        self.assertEqual(self.money.format('de_DE'), '-1.234,57\xa0$')
        self.assertEqual(self.money.format('es_CO'), '-1.234,57\xa0US$')
    
    def test_auto_format_locales_alias(self):
        self.assertEqual(self.money.format('en'), self.money.format('en_US'))
        self.assertEqual(self.money.format('de'), self.money.format('de_DE'))


class ParserMixin(object):
    def test_loads_valid(self):
        self.assertEqual(self.MoneyClass.loads('XXX 2.22'), self.MoneyClass('2.22', 'XXX'))
    
    def test_loads_missing_currency(self):
        with self.assertRaises(ValueError):
            money = self.MoneyClass.loads('2.22')
    
    def test_loads_reversed_order(self):
        with self.assertRaises(ValueError):
            money = self.MoneyClass.loads('2.22 XXX')
    
    def test_loads_empty(self):
        with self.assertRaises(ValueError):
            money = self.MoneyClass.loads('')


class NumericOperationsMixin(object):
    def test_lt_number(self):
        self.assertTrue(self.MoneyClass('2.22', 'XXX') < 3)
        self.assertTrue(self.MoneyClass('2.22', 'XXX') < 3.0)
        self.assertTrue(self.MoneyClass('2.22', 'XXX') < Decimal(3))
    
    def test_lt_money(self):
        self.assertTrue(self.MoneyClass('2.219', 'XXX') < self.MoneyClass('2.22', 'XXX'))
        self.assertTrue(self.MoneyClass('-2.22', 'XXX') < self.MoneyClass('2.22', 'XXX'))
    
    def test_lt_none(self):
        with self.assertRaises(TypeError):
            self.MoneyClass(0, 'XXX') < None
    
    def test_le_number(self):
        self.assertTrue(self.MoneyClass('2.219', 'XXX') <= 3)
        self.assertTrue(self.MoneyClass('2.219', 'XXX') <= 3.0)
        self.assertTrue(self.MoneyClass('-2.22', 'XXX') <= Decimal('3'))
    
    def test_le_money(self):
        self.assertTrue(self.MoneyClass('2.219', 'XXX') <= self.MoneyClass('2.22', 'XXX'))
        self.assertTrue(self.MoneyClass('-2.22', 'XXX') <= self.MoneyClass('2.22', 'XXX'))
        self.assertTrue(self.MoneyClass('2.220', 'XXX') <= self.MoneyClass('2.22', 'XXX'))
    
    def test_le_none(self):
        with self.assertRaises(TypeError):
            self.MoneyClass(0, 'XXX') <= None
    
    def test_eq(self):
        self.assertEqual(self.MoneyClass('2', 'XXX'), self.MoneyClass('2', 'XXX'))
        self.assertEqual(self.MoneyClass('2.22000', 'XXX'), self.MoneyClass('2.22', 'XXX'))
    
    def test_ne(self):
        self.assertNotEqual(self.MoneyClass('0', 'XXX'), 0)
        self.assertNotEqual(self.MoneyClass('2', 'XXX'), 2)
        self.assertNotEqual(self.MoneyClass('2', 'XXX'), 'two')
    
    def test_ne_money(self):
        self.assertNotEqual(self.MoneyClass('2', 'XXX'), self.MoneyClass('3', 'XXX'))
        self.assertNotEqual(self.MoneyClass('2', 'XXX'), self.MoneyClass('2', 'YYY'))
    
    def test_ne_none(self):
        self.assertNotEqual(self.MoneyClass(0, 'XXX'), None)
    
    def test_gt_number(self):
        self.assertTrue(self.MoneyClass('2.22', 'XXX') > 2)
        self.assertTrue(self.MoneyClass('2.22', 'XXX') > Decimal('2'))
    
    def test_gt_money(self):
        self.assertTrue(self.MoneyClass('2.22', 'XXX') > self.MoneyClass('2.219', 'XXX'))
        self.assertTrue(self.MoneyClass('2.22', 'XXX') > self.MoneyClass('-2.22', 'XXX'))
    
    def test_gt_none(self):
        with self.assertRaises(TypeError):
            self.MoneyClass(0, 'XXX') > None
    
    def test_ge_number(self):
        self.assertTrue(self.MoneyClass('2', 'XXX') >= 1)
        self.assertTrue(self.MoneyClass('2', 'XXX') >= 2)
        self.assertTrue(self.MoneyClass('2', 'XXX') >= Decimal('1'))
        self.assertTrue(self.MoneyClass('2', 'XXX') >= Decimal('2'))
    
    def test_ge_money(self):
        self.assertTrue(self.MoneyClass('2.22', 'XXX') >= self.MoneyClass('2.219', 'XXX'))
        self.assertTrue(self.MoneyClass('2.22', 'XXX') >= self.MoneyClass('-2.22', 'XXX'))
        self.assertTrue(self.MoneyClass('2.22', 'XXX') >= self.MoneyClass('2.22', 'XXX'))
    
    def test_ge_none(self):
        with self.assertRaises(TypeError):
            self.MoneyClass(0, 'XXX') >= None
    
    def test_bool_true(self):
        self.assertTrue(self.MoneyClass('2.22', 'XXX'))
        self.assertTrue(self.MoneyClass('-1', 'XXX'))
    
    def test_bool_false(self):
        self.assertFalse(self.MoneyClass('0', 'XXX'))
    
    def test_add_int(self):
        result = self.MoneyClass('2', 'XXX') + 2
        self.assertEqual(result, self.MoneyClass('4', 'XXX'))
    
    def test_add_decimal(self):
        result = self.MoneyClass('2', 'XXX') + Decimal('2')
        self.assertEqual(result, self.MoneyClass('4', 'XXX'))
    
    def test_add_money(self):
        result = self.MoneyClass('2', 'XXX') + self.MoneyClass('2', 'XXX')
        self.assertEqual(result, self.MoneyClass('4', 'XXX'))
    
    def test_add_none(self):
        with self.assertRaises(TypeError):
            self.MoneyClass(0, 'XXX') + None
    
    def test_radd_int(self):
        result = 2 + self.MoneyClass('2', 'XXX')
        self.assertEqual(result, self.MoneyClass('4', 'XXX'))
    
    def test_sub_int(self):
        result = self.MoneyClass('2', 'XXX') - 2
        self.assertEqual(result, self.MoneyClass('0', 'XXX'))
    
    def test_sub_decimal(self):
        result = self.MoneyClass('2', 'XXX') - Decimal(2)
        self.assertEqual(result, self.MoneyClass('0', 'XXX'))
    
    def test_sub_money(self):
        result = self.MoneyClass('2', 'XXX') - self.MoneyClass('2', 'XXX')
        self.assertEqual(result, self.MoneyClass('0', 'XXX'))
    
    def test_sub_none(self):
        with self.assertRaises(TypeError):
            self.MoneyClass(0, 'XXX') - None
    
    def test_rsub_int(self):
        result = 0 - self.MoneyClass('2', 'XXX')
        self.assertEqual(result, self.MoneyClass('-2', 'XXX'))
    
    def test_mul_int(self):
        result = self.MoneyClass('2', 'XXX') * 2
        self.assertEqual(result, self.MoneyClass('4', 'XXX'))
    
    def test_mul_decimal(self):
        result = self.MoneyClass('2', 'XXX') * Decimal(2)
        self.assertEqual(result, self.MoneyClass('4', 'XXX'))
    
    def test_mul_money(self):
        with self.assertRaises(TypeError):
            self.MoneyClass('2', 'XXX') * self.MoneyClass('2', 'XXX')
    
    def test_mul_none(self):
        with self.assertRaises(TypeError):
            self.MoneyClass(0, 'XXX') * None
    
    def test_rmul_int(self):
        result = 2 * self.MoneyClass('2', 'XXX')
        self.assertEqual(result, self.MoneyClass('4', 'XXX'))
    
    def test_truediv_int(self):
        result = self.MoneyClass('2.22', 'XXX') / 2
        self.assertEqual(result, self.MoneyClass('1.11', 'XXX'))
    
    def test_truediv_decimal(self):
        result = self.MoneyClass('2.22', 'XXX') / Decimal(2)
        self.assertEqual(result, self.MoneyClass('1.11', 'XXX'))
    
    def test_truediv_money(self):
        result = self.MoneyClass('2', 'XXX') / self.MoneyClass('2', 'XXX')
        self.assertEqual(result, Decimal('1'))
    
    def test_truediv_none(self):
        with self.assertRaises(TypeError):
            self.MoneyClass(2, 'XXX') / None
    
    def test_truediv_zero(self):
        with self.assertRaises(ZeroDivisionError):
            self.MoneyClass(2, 'XXX') / 0
    
    def test_floordiv_number(self):
        result = self.MoneyClass('2.22', 'XXX') // 2
        self.assertEqual(result, self.MoneyClass('1', 'XXX'))
    
    def test_floordiv_money(self):
        result = self.MoneyClass('2.22', 'XXX') // self.MoneyClass('2', 'XXX')
        self.assertEqual(result, Decimal('1'))
    
    def test_floordiv_none(self):
        with self.assertRaises(TypeError):
            self.MoneyClass(2, 'XXX') // None
    
    def test_floordiv_zero(self):
        with self.assertRaises(ZeroDivisionError):
            self.MoneyClass(2, 'XXX') // 0
    
    def test_mod_number(self):
        result = self.MoneyClass('2.22', 'XXX') % 2
        self.assertEqual(result, self.MoneyClass('0.22', 'XXX'))
    
    def test_mod_money(self):
        with self.assertRaises(TypeError):
            self.MoneyClass('2.22', 'XXX') % self.MoneyClass('2', 'XXX')
    
    def test_mod_none(self):
        with self.assertRaises(TypeError):
            self.MoneyClass(2, 'XXX') % None
    
    def test_mod_zero(self):
        with self.assertRaises(ZeroDivisionError):
            self.MoneyClass(2, 'XXX') % 0
    
    def test_divmod_number(self):
        whole, remainder = divmod(self.MoneyClass('2.22', 'XXX'), 2)
        self.assertEqual(whole, self.MoneyClass('1', 'XXX'))
        self.assertEqual(remainder, self.MoneyClass('0.22', 'XXX'))
    
    def test_divmod_money(self):
        whole, remainder = divmod(self.MoneyClass('2.22', 'XXX'), self.MoneyClass('2', 'XXX'))
        self.assertEqual(whole, Decimal('1'))
        self.assertEqual(remainder, Decimal('0.22'))
    
    def test_divmod_none(self):
        with self.assertRaises(TypeError):
            divmod(self.MoneyClass(2, 'XXX'), None)
    
    def test_divmod_zero(self):
        with self.assertRaises(ZeroDivisionError):
            divmod(self.MoneyClass(2, 'XXX'), 0)
    
    def test_pow_number(self):
        result = self.MoneyClass('3', 'XXX') ** 2
        self.assertEqual(result, self.MoneyClass('9', 'XXX'))
    
    def test_pow_money(self):
        with self.assertRaises(TypeError):
            self.MoneyClass('3', 'XXX') ** self.MoneyClass('2', 'XXX')
    
    def test_pow_none(self):
        with self.assertRaises(TypeError):
            self.MoneyClass(0, 'XXX') ** None
    
    def test_neg(self):
        result = -self.MoneyClass('2.22', 'XXX')
        self.assertEqual(result, self.MoneyClass('-2.22', 'XXX'))
    
    def test_pos(self):
        result = +self.MoneyClass('2.22', 'XXX')
        self.assertEqual(result, self.MoneyClass('2.22', 'XXX'))
    
    def test_abs(self):
        result = abs(self.MoneyClass('-2.22', 'XXX'))
        self.assertEqual(result, self.MoneyClass('2.22', 'XXX'))
    
    def test_int(self):
        self.assertEqual(int(self.MoneyClass('-2.22', 'XXX')), -2)
        self.assertEqual(int(self.MoneyClass('2.22', 'XXX')), 2)
    
    def test_float(self):
        self.assertEqual(float(self.MoneyClass('-2.22', 'XXX')), -2.22)
        self.assertEqual(float(self.MoneyClass('2.22', 'XXX')), 2.22)
    
    def test_round(self):
        self.assertEqual(round(self.MoneyClass('-1.49', 'XXX')), self.MoneyClass('-1', 'XXX'))
        self.assertEqual(round(self.MoneyClass('1.50', 'XXX')), self.MoneyClass('2', 'XXX'))
        self.assertEqual(round(self.MoneyClass('1.234', 'XXX'), 2), self.MoneyClass('1.23', 'XXX'))


class UnaryOperationsReturnNewMixin(object):
    def setUp(self):
        self.money = self.MoneyClass(2, 'XXX')
    
    def test_pos(self):
        self.assertIsNot(+self.money, self.money)
    
    def test_abs(self):
        self.assertIsNot(abs(self.money), self.money)
    
    def test_round(self):
        self.assertIsNot(round(self.money), self.money)
    
    def test_sqlalchemy_composite_values(self):
        self.assertEqual((Decimal(2), 'XXX'), self.money.__composite_values__())


class LeftmostTypePrevailsMixin(object):
    def setUp(self):
        if self.MoneyClass.__name__ == 'Money':
            self.OtherClass = XMoney
        if self.MoneyClass.__name__ == 'XMoney':
            self.OtherClass = Money
        
        self.home = self.MoneyClass(2, 'XXX')
        self.visitor = self.OtherClass(2, 'XXX')
    
    def test_add(self):
        result = self.home + self.visitor
        self.assertEqual(result.__class__, self.MoneyClass)
    
    def test_add_other(self):
        result = self.visitor + self.home
        self.assertEqual(result.__class__, self.OtherClass)
    
    def test_sub(self):
        result = self.home - self.visitor
        self.assertEqual(result.__class__, self.MoneyClass)
    
    def test_sub_other(self):
        result = self.visitor - self.home
        self.assertEqual(result.__class__, self.OtherClass)
    










# -*- coding: utf-8 -*-
"""
XMoney class unittests
"""
from decimal import Decimal
import unittest

from money import XMoney, xrates
from money.money import BABEL_AVAILABLE
from .mixins import *


class TestXMoneyClass(ClassMixin, unittest.TestCase):
    MoneyClass = XMoney

class TestXMoneyRepresentations(RepresentationsMixin, unittest.TestCase):
    MoneyClass = XMoney

@unittest.skipUnless(BABEL_AVAILABLE, "requires Babel")
class TestXMoneyFormatting(FormattingMixin, unittest.TestCase):
    MoneyClass = XMoney

class TestXMoneyParser(ParserMixin, unittest.TestCase):
    MoneyClass = XMoney

class TestXMoneyNumericOperations(NumericOperationsMixin, unittest.TestCase):
    MoneyClass = XMoney

class TestXMoneyUnaryOperationsReturnNew(UnaryOperationsReturnNewMixin, unittest.TestCase):
    MoneyClass = XMoney

class TestXMoneyLeftmostTypePrevails(LeftmostTypePrevailsMixin, unittest.TestCase):
    MoneyClass = XMoney


class TestXMoneyAutoConversion(unittest.TestCase):
    @classmethod
    def setUpClass(cls):
        xrates.install('money.exchange.SimpleBackend')
        xrates.base = 'XXX'
        xrates.setrate('AAA', Decimal('2'))
        xrates.setrate('BBB', Decimal('8'))
    
    @classmethod
    def tearDownClass(cls):
        xrates.uninstall()
    
    def setUp(self):
        self.x = XMoney('10', 'XXX')
        self.a = XMoney('10', 'AAA')
        self.b = XMoney('10', 'BBB')
        self.ax = XMoney('20', 'AAA')
        self.bx = XMoney('80', 'BBB')
    
    def test_lt(self):
        self.assertTrue(self.b < self.a)
        self.assertFalse(self.a < self.b)
        self.assertFalse(self.ax < self.bx)
    
    def test_le(self):
        self.assertTrue(self.b <= self.a)
        self.assertFalse(self.a <= self.b)
        self.assertTrue(self.ax <= self.bx)
        self.assertTrue(self.bx <= self.ax)
    
    def test_gt(self):
        self.assertTrue(self.a > self.b)
        self.assertFalse(self.b > self.a)
        self.assertFalse(self.ax > self.bx)
    
    def test_ge(self):
        self.assertTrue(self.a >= self.b)
        self.assertFalse(self.b >= self.a)
        self.assertTrue(self.ax >= self.bx)
        self.assertTrue(self.bx >= self.ax)
    
    def test_add(self):
        self.assertEqual(self.a + self.b, XMoney('12.5', 'AAA'))
        self.assertEqual(self.b + self.a, XMoney('50', 'BBB'))
    
    def test_sub(self):
        self.assertEqual(self.a - self.b, XMoney('7.5', 'AAA'))
        self.assertEqual(self.b - self.a, XMoney('-30', 'BBB'))
    
    def test_truediv(self):
        self.assertEqual(self.a / self.b, Decimal('4'))
        self.assertEqual(self.b / self.a, Decimal('0.25'))
    
    def test_floordiv(self):
        self.assertEqual(self.a // self.b, Decimal('4'))
        self.assertEqual(self.b // self.a, Decimal('0'))
    
    def test_divmod(self):
        whole, remainder = divmod(self.a, self.b)
        self.assertEqual(whole, Decimal('4'))
        self.assertEqual(remainder, Decimal('0'))
        whole, remainder = divmod(self.b, self.a)
        self.assertEqual(whole, Decimal('0'))
        self.assertEqual(remainder, Decimal('10'))






# -*- coding: utf-8 -*-
"""
Money classes
"""
import decimal
import re
from distutils.version import StrictVersion

from .exchange import xrates
from .exceptions import CurrencyMismatch, ExchangeRateNotFound


__all__ = ['Money', 'XMoney']

BABEL_AVAILABLE = False
BABEL_VERSION = None
REGEX_CURRENCY_CODE = re.compile("^[A-Z]{3}$")

try:
    import babel
    import babel.numbers
    BABEL_AVAILABLE = True
    BABEL_VERSION = StrictVersion(babel.__version__)
except ImportError:
    pass


class Money(object):
    """Money class with a decimal amount and a currency"""
    __hash__ = None
    
    def __init__(self, amount="0", currency=None):
        try:
            self.amount = decimal.Decimal(amount)
        except decimal.InvalidOperation:
            raise ValueError("amount value could not be converted to "
                             "Decimal(): '{}'".format(amount))
        if currency in [None, False, '']:
            raise ValueError("invalid currency value: '{}'".format(currency))
        if not REGEX_CURRENCY_CODE.match(currency):
            raise ValueError("currency not in ISO 4217 format: "
                             "'{}'".format(currency))
        self.currency = currency
    
    def __repr__(self):
        return "{} {}".format(self.currency, self.amount)
    
    def __str__(self):
        return self.__unicode__().encode('utf-8')
    
    def __unicode__(self):
        return u"{} {:,.2f}".format(self.currency, self.amount)
    
    def __lt__(self, other):
        if isinstance(other, Money):
            if other.currency != self.currency:
                raise CurrencyMismatch(self.currency, other.currency, '<')
            other = other.amount
        return self.amount < other
    
    def __le__(self, other):
        if isinstance(other, Money):
            if other.currency != self.currency:
                raise CurrencyMismatch(self.currency, other.currency, '<=')
            other = other.amount
        return self.amount <= other
    
    def __eq__(self, other):
        if isinstance(other, Money):
            return ((self.amount == other.amount) and
                    (self.currency == other.currency))
        return False
    
    def __ne__(self, other):
        return not self == other
    
    def __gt__(self, other):
        if isinstance(other, Money):
            if other.currency != self.currency:
                raise CurrencyMismatch(self.currency, other.currency, '>')
            other = other.amount
        return self.amount > other
    
    def __ge__(self, other):
        if isinstance(other, Money):
            if other.currency != self.currency:
                raise CurrencyMismatch(self.currency, other.currency, '>=')
            other = other.amount
        return self.amount >= other
    
    def __nonzero__(self):
        """
        Considering Money a numeric type (on ``amount``):
        
        bool(Money(2, 'XXX')) --> True
        bool(Money(0, 'XXX')) --> False
        """
        return bool(self.amount)
    
    def __add__(self, other):
        if isinstance(other, Money):
            if other.currency != self.currency:
                raise CurrencyMismatch(self.currency, other.currency, '+')
            other = other.amount
        amount = self.amount + other
        return self.__class__(amount, self.currency)
    
    def __radd__(self, other):
        return self.__add__(other)
    
    def __sub__(self, other):
        if isinstance(other, Money):
            if other.currency != self.currency:
                raise CurrencyMismatch(self.currency, other.currency, '-')
            other = other.amount
        amount = self.amount - other
        return self.__class__(amount, self.currency)
    
    def __rsub__(self, other):
        return (-self).__add__(other)
    
    def __mul__(self, other):
        if isinstance(other, Money):
            raise TypeError("multiplication is unsupported between "
                            "two money objects")
        amount = self.amount * other
        return self.__class__(amount, self.currency)
    
    def __rmul__(self, other):
        return self.__mul__(other)
    
    def __div__(self, other):
        return self.__truediv__(other)
    
    def __truediv__(self, other):
        if isinstance(other, Money):
            if other.currency != self.currency:
                raise CurrencyMismatch(self.currency, other.currency, '/')
            elif other.amount == 0:
                raise ZeroDivisionError()
            return self.amount / other.amount
        else:
            if other == 0:
                raise ZeroDivisionError()
            amount = self.amount / other
            return self.__class__(amount, self.currency)
    
    def __floordiv__(self, other):
        if isinstance(other, Money):
            if other.currency != self.currency:
                raise CurrencyMismatch(self.currency, other.currency, '//')
            elif other.amount == 0:
                raise ZeroDivisionError()
            return self.amount // other.amount
        else:
            if other == 0:
                raise ZeroDivisionError()
            amount = self.amount // other
            return self.__class__(amount, self.currency)
    
    def __mod__(self, other):
        if isinstance(other, Money):
            raise TypeError("modulo is unsupported between two '{}' "
                            "objects".format(self.__class__.__name__))
        if other == 0:
            raise ZeroDivisionError()
        amount = self.amount % other
        return self.__class__(amount, self.currency)
    
    def __divmod__(self, other):
        if isinstance(other, Money):
            if other.currency != self.currency:
                raise CurrencyMismatch(self.currency, other.currency, 'divmod')
            elif other.amount == 0:
                raise ZeroDivisionError()
            return divmod(self.amount, other.amount)
        else:
            if other == 0:
                raise ZeroDivisionError()
            whole, remainder = divmod(self.amount, other)
            return (self.__class__(whole, self.currency),
                    self.__class__(remainder, self.currency))
    
    def __pow__(self, other):
        if isinstance(other, Money):
            raise TypeError("power operator is unsupported between two '{}' "
                            "objects".format(self.__class__.__name__))
        amount = self.amount ** other
        return self.__class__(amount, self.currency)
    
    def __neg__(self):
        return self.__class__(-self.amount, self.currency)
    
    def __pos__(self):
        return self.__class__(+self.amount, self.currency)
    
    def __abs__(self):
        return self.__class__(abs(self.amount), self.currency)
        
    def __int__(self):
        return int(self.amount)
    
    def __float__(self):
        return float(self.amount)
    
    def __round__(self, ndigits=0):
        return self.__class__(round(self.amount, ndigits), self.currency)
    
    def __composite_values__(self):
        return self.amount, self.currency
    
    def to(self, currency):
        """Return equivalent money object in another currency"""
        if currency == self.currency:
            return self
        rate = xrates.quotation(self.currency, currency)
        if rate is None:
            raise ExchangeRateNotFound(xrates.backend_name,
                                         self.currency, currency)
        amount = self.amount * rate
        return self.__class__(amount, currency)
    
    def format(self, locale=None, pattern=None):
        """
        Return a locale-aware, currency-formatted string.
        
        This method emulates babel.numbers.format_currency().
        
        A specific locale identifier (language[_territory]) can be passed,
        otherwise the system's default locale will be used. A custom
        formatting pattern of the form "¤#,##0.00;(¤#,##0.00)"
        (positive[;negative]) can also be passed, otherwise it will be
        determined from the locale and the CLDR (Unicode Common Locale Data
        Repository) included with Babel.
        
        >>> m = Money('1234.567', 'EUR')
        >>> m.format() # assuming the system's locale is 'en_US'
        €1,234.57
        >>> m.format('de_DE') # German formatting
        1.234,57 €
        >>> m.format('de', '#,##0 ¤') # German formatting (short), no cents
        1.235 €
        >>> m.format(pattern='#,##0.00 ¤¤¤') # Default locale, full name
        1,235.57 euro
        
        Learn more about this formatting syntaxis at:
        http://www.unicode.org/reports/tr35/tr35-numbers.html
        """
        if BABEL_AVAILABLE:
            if BABEL_VERSION >= StrictVersion('2.2'):
                raise Exception('Babel {} is unsupported. '
                    'Please upgrade Money to 1.3 or higher.'.format(
                    BABEL_VERSION))
            if not locale:
                locale = babel.default_locale('LC_NUMERIC')
            locale = babel.Locale.parse(locale)
            if not pattern:
                pattern = locale.currency_formats.get(pattern)
            pattern = babel.numbers.parse_pattern(pattern)
            return pattern.apply(self.amount, locale, currency=self.currency)
        else:
            raise NotImplementedError("formatting requires Babel "
                                      "(https://pypi.python.org/pypi/Babel)")
    
    @classmethod
    def loads(cls, s):
        """Parse from a string representation (repr)"""
        try:
            currency, amount = s.strip().split(' ')
            return cls(amount, currency)
        except ValueError as err:
            raise ValueError("failed to parse string '{}': "
                             "{}".format(s, err))


class XMoney(Money):
    """Money subclass with implicit currency conversion"""
    def __lt__(self, other):
        if isinstance(other, Money):
            other = other.to(self.currency)
        return super(XMoney, self).__lt__(other)
    
    def __le__(self, other):
        if isinstance(other, Money):
            other = other.to(self.currency)
        return super(XMoney, self).__le__(other)
    
    def __gt__(self, other):
        if isinstance(other, Money):
            other = other.to(self.currency)
        return super(XMoney, self).__gt__(other)
    
    def __ge__(self, other):
        if isinstance(other, Money):
            other = other.to(self.currency)
        return super(XMoney, self).__ge__(other)
    
    def __add__(self, other):
        if isinstance(other, Money):
            other = other.to(self.currency)
        return super(XMoney, self).__add__(other)
    
    def __sub__(self, other):
        if isinstance(other, Money):
            other = other.to(self.currency)
        return super(XMoney, self).__sub__(other)
    
    def __div__(self, other):
        return self.__truediv__(other)
    
    def __truediv__(self, other):
        if isinstance(other, Money):
            other = other.to(self.currency)
        return super(XMoney, self).__truediv__(other)
    
    def __floordiv__(self, other):
        if isinstance(other, Money):
            other = other.to(self.currency)
        return super(XMoney, self).__floordiv__(other)
    
    def __divmod__(self, other):
        if isinstance(other, Money):
            other = other.to(self.currency)
        return super(XMoney, self).__divmod__(other)










# -*- coding: utf-8 -*-
"""
Python money class with optional CLDR-backed locale-aware formatting
and an extensible currency exchange solution.
"""
from .money import Money, XMoney
from .exchange import xrates


__version__ = '1.2.3'



# -*- coding: utf-8 -*-
"""
Money class unittests
"""
import unittest

from money import Money
from money.money import BABEL_AVAILABLE
from .mixins import *


class TestMoneyClass(ClassMixin, unittest.TestCase):
    MoneyClass = Money

class TestMoneyRepresentations(RepresentationsMixin, unittest.TestCase):
    MoneyClass = Money

@unittest.skipUnless(BABEL_AVAILABLE, "requires Babel")
class TestMoneyFormatting(FormattingMixin, unittest.TestCase):
    MoneyClass = Money

class TestMoneyParser(ParserMixin, unittest.TestCase):
    MoneyClass = Money

class TestMoneyNumericOperations(NumericOperationsMixin, unittest.TestCase):
    MoneyClass = Money

class TestMoneyUnaryOperationsReturnNew(UnaryOperationsReturnNewMixin, unittest.TestCase):
    MoneyClass = Money

class TestMoneyLeftmostTypePrevails(LeftmostTypePrevailsMixin, unittest.TestCase):
    MoneyClass = Money



import sys
from distutils.core import setup


DESCRIPTION = """
Python money class with optional CLDR-backed locale-aware formatting 
and an extensible currency exchange solution.
"""

SOURCE_ROOT = 'src'

# Python 2 backwards compatibility
if sys.version_info[0] == 2:
    SOURCE_ROOT = 'src-py2'


setup(
    name='money',
    description='Python Money Class',
    # long_description=DESCRIPTION,
    version='1.2.3',
    author='Carlos Palol',
    author_email='carlos.palol@awarepixel.com',
    url='https://github.com/carlospalol/money',
    license='MIT',
    package_dir={'': SOURCE_ROOT},
    packages=[
        'money',
    ],
    classifiers=[
        'Development Status :: 4 - Beta',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Programming Language :: Python :: 2',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.3',
        'Programming Language :: Python :: 3.4',
        'Topic :: Software Development :: Libraries',
    ]
)



# -*- coding: utf-8 -*-
"""
Money exceptions
"""


class CurrencyMismatch(Exception):
    """Invalid operation between money objects of different currencies"""
    def __init__(self, a, b, operation):
        msg = ("unsupported operation between money in '{}' and '{}': '{}'. "
               "Use XMoney for automatic currency conversion."
               ).format(a, b, operation)
        super(CurrencyMismatch, self).__init__(msg)


class ExchangeError(Exception):
    """Generic exception related to exchange rates"""
    pass


class ExchangeBackendNotInstalled(ExchangeError):
    """No backend installed yet"""
    def __init__(self):
        msg = "use e.g. money.xrates.install('money.exchange.SimpleBackend')"
        super(ExchangeBackendNotInstalled, self).__init__(msg)


class ExchangeRateNotFound(ExchangeError):
    """A rate/quotation was not returned by the backend"""
    def __init__(self, backend, a, b):
        msg = ("rate not found in backend '{}': {}/{}".format(backend, a, b))
        super(ExchangeRateNotFound, self).__init__(msg)


